==> exframe-service-1.5.0.tgz: package/index.js <==
'use strict';

const health = require('exframe-health');
const prometheusClient = require('prom-client');

const MemoryWatcher = require('./lib/memory-watcher');

const loggerDefault = {
  /* eslint no-console: ["error", { allow: ["info", "log", "error", "warn"] }] */
  info: (x) => console.info(x),
  debug: (x) => console.log(x),
  error: (x) => console.error(x),
  warn: (x) => console.warn(x),
  silly: (x) => console.log(x),
  flushLogs: (callback) => callback()
};

const STATUS = {
  RUNNING: 'Running',
  SHUTTING_DOWN: 'ShuttingDown'
};

const serviceStatus = {
  status: STATUS.RUNNING,
  resources: []
};

const settings = {
  logger: loggerDefault,
  timeout: 10000,
  memoryWatcher: null
};

const initializeMemoryWatcher = (options) => {
  settings.memoryWatcher = MemoryWatcher.getInstance(options);
  settings.memoryWatcher.start();
  health.liveness.add('memory', async () => (settings.memoryWatcher.liveness
    ? { status: 200, message: 'OK' }
    : { status: 503, message: 'Liveness memory limit exceeded' }));

  health.readiness.add('memory', async () => (settings.memoryWatcher.readiness
    ? { status: 200, message: 'OK' }
    : { status: 503, message: 'Readiness memory limit exceeded' }));
};

const init = (options = {}) => {
  settings.logger = options.logger || loggerDefault;
  settings.timeout = options.timeout;

  initializeMemoryWatcher({ ...options.memoryWatcher, logger: settings.logger });
};

const registerResource = (name, { onSignal, order = 'general' }) => {
  const resourceStatus = {
    name: '',
    status: STATUS.RUNNING,
    signal: {},
    order
  };

  resourceStatus.name = name;
  resourceStatus.signal = onSignal;
  resourceStatus.order = order.toLowerCase();
  serviceStatus.resources.push(resourceStatus);
  settings.logger.info(`registered service: ${name}.`);
};

const sortResources = () => {
  const sortedResources = serviceStatus.resources.sort((a, b) => a.order.localeCompare(b.order));
  return sortedResources;
};

const runOnSignal = async () => {
  let generalResult = [];
  let lastResult = [];
  const getGeneralSignal = serviceStatus.resources.filter((resource) => resource.order === 'general');
  const getLastSignal = serviceStatus.resources.filter((resource) => resource.order === 'last');
  if (getGeneralSignal.length > 0) {
    generalResult = await Promise.all(getGeneralSignal.map((r) => r.signal()));
  }

  if (getLastSignal.length > 0) {
    lastResult = await Promise.all(getLastSignal.map((r) => r.signal()));
  }

  return [...generalResult, ...lastResult];
};

const gracefulShutdown = async () => {
  try {
    const sortedResources = await sortResources();
    const result = await runOnSignal();
    for (let i = 0; i < sortedResources.length; i++) {
      settings.logger.info(`${serviceStatus.resources[i].name} onSignal result = ${result[i]}`);
    }
    settings.logger.info('process exiting.');
    settings.logger.flushLogs(() => {
      process.exit(); // eslint-disable-line no-process-exit
    });
  } catch (ex) {
    settings.logger.info('Error occurred running gracefulShutdown', { errorData: ex });
  }
};

const onSigterm = async () => {
  if (serviceStatus.status !== STATUS.RUNNING) return;
  settings.logger.info('SIGTERM signal received.');
  settings.logger.info('shutting down.');
  serviceStatus.status = STATUS.SHUTTING_DOWN;
  setTimeout(() => {
    gracefulShutdown();
  }, settings.timeout);
  settings.logger.info(`process waiting for ${settings.timeout}ms before exiting`);
};

const main = () => {
  process.on('SIGTERM', onSigterm);

  health.liveness.add('service', async () => ((serviceStatus.status === STATUS.RUNNING)
    ? { status: 200, message: 'OK' }
    : { status: 503, message: 'Shutting Down', resources: serviceStatus.resources }
  ));

  prometheusClient.register.setDefaultLabels({ serviceName: process.env.npm_package_name });
  try {
    prometheusClient.collectDefaultMetrics();
  } catch (error) {
    // The prom-client module expects singleton usage and unfortunately "collectDefaultMetrics" is non-idempotent,
    // so we're ignoring errors for default metrics already registered which will really only occur in tests if this
    // module is forcibly loaded multiple times in the same process, which would never happen during normal execution
    // in a service because of the require cache.
    if (!/A metric with the name .* has already been registered./.test(error.message)) {
      throw error;
    }
  }
};

main();

module.exports = {
  init,
  registerResource,
  onSigterm,
  gracefulShutdown,
  settings,
  serviceStatus,
  process,
  STATUS,
  runOnSignal,
  sortResources,
  prometheusClient
};

==> exframe-service-1.6.2.tgz: package/index.js <==
'use strict';

const health = require('exframe-health');
const prometheusClient = require('prom-client');
const { getPackageName } = require('exframe-utilities');

const MemoryWatcher = require('./lib/memory-watcher');

const loggerDefault = {
  /* eslint no-console: ["error", { allow: ["info", "log", "error", "warn"] }] */
  info: (x) => console.info(x),
  debug: (x) => console.log(x),
  error: (x) => console.error(x),
  warn: (x) => console.warn(x),
  silly: (x) => console.log(x),
  flushLogs: (callback) => callback()
};

const STATUS = {
  RUNNING: 'Running',
  SHUTTING_DOWN: 'ShuttingDown'
};

const serviceStatus = {
  status: STATUS.RUNNING,
  resources: []
};

const settings = {
  logger: loggerDefault,
  timeout: 10000,
  memoryWatcher: null
};

const registerResource = (name, { onSignal, order = 'general' }) => {
  const resourceStatus = {
    name: '',
    status: STATUS.RUNNING,
    signal: {},
    order
  };

  resourceStatus.name = name;
  resourceStatus.signal = onSignal;
  resourceStatus.order = order.toLowerCase();
  serviceStatus.resources.push(resourceStatus);
  settings.logger.info(`registered service: ${name}.`);
};

const sortResources = () => {
  const sortedResources = serviceStatus.resources.sort((a, b) => a.order.localeCompare(b.order));
  return sortedResources;
};

const runOnSignal = async () => {
  let generalResult = [];
  let lastResult = [];
  const getGeneralSignal = serviceStatus.resources.filter((resource) => resource.order === 'general');
  const getLastSignal = serviceStatus.resources.filter((resource) => resource.order === 'last');
  if (getGeneralSignal.length > 0) {
    generalResult = await Promise.all(getGeneralSignal.map((r) => r.signal()));
  }

  if (getLastSignal.length > 0) {
    lastResult = await Promise.all(getLastSignal.map((r) => r.signal()));
  }

  return [...generalResult, ...lastResult];
};

const gracefulShutdown = async () => {
  try {
    const sortedResources = await sortResources();
    const result = await runOnSignal();
    for (let i = 0; i < sortedResources.length; i++) {
      settings.logger.info(`${serviceStatus.resources[i].name} onSignal result = ${result[i]}`);
    }
    settings.logger.info('process exiting.');
    settings.logger.flushLogs(() => {
      process.exit(); // eslint-disable-line no-process-exit
    });
  } catch (ex) {
    settings.logger.info('Error occurred running gracefulShutdown', { errorData: ex });
  }
};

const onSigterm = async () => {
  if (serviceStatus.status !== STATUS.RUNNING) return;
  settings.logger.info('SIGTERM signal received.');
  settings.logger.info('shutting down.');
  serviceStatus.status = STATUS.SHUTTING_DOWN;
  setTimeout(() => {
    gracefulShutdown();
  }, settings.timeout);
  settings.logger.info(`process waiting for ${settings.timeout}ms before exiting`);
};

const initializeMemoryWatcher = (options) => {
  settings.memoryWatcher = MemoryWatcher.getInstance(options);
  settings.memoryWatcher.start();

  health.add('memory-watcher', async (probeType) => {
    const isHealthy = probeType === health.ProbeTypes.Readiness
      ? settings.memoryWatcher.readiness
      : settings.memoryWatcher.liveness;

    return isHealthy
      ? { status: 200, message: 'OK' }
      : { status: 503, message: `${probeType} memory limit exceeded` };
  }, { promotionType: health.PromotionTypes.UserControlled });

  registerResource('memory-watcher', {
    onSignal: async () => {
      settings.memoryWatcher.stop();
      return true;
    }
  });
};

const init = (options = {}) => {
  settings.logger = options.logger || loggerDefault;
  settings.timeout = options.timeout;

  initializeMemoryWatcher({ ...options.memoryWatcher, logger: settings.logger });
};

const main = () => {
  process.on('SIGTERM', onSigterm);

  health.add('service', async () => ((serviceStatus.status === STATUS.RUNNING)
    ? { status: 200, message: 'OK' }
    : { status: 503, message: 'Shutting Down', resources: serviceStatus.resources }
  ));

  prometheusClient.register.setDefaultLabels({ serviceName: getPackageName() });
  try {
    prometheusClient.collectDefaultMetrics();
  } catch (error) {
    // The prom-client module expects singleton usage and unfortunately "collectDefaultMetrics" is non-idempotent,
    // so we're ignoring errors for default metrics already registered which will really only occur in tests if this
    // module is forcibly loaded multiple times in the same process, which would never happen during normal execution
    // in a service because of the require cache.
    if (!/A metric with the name .* has already been registered./.test(error.message)) {
      throw error;
    }
  }
};

main();

module.exports = {
  init,
  registerResource,
  onSigterm,
  gracefulShutdown,
  settings,
  serviceStatus,
  process,
  STATUS,
  runOnSignal,
  sortResources,
  prometheusClient
};
